       SUBROUTINE tqli(d,e,n,np,z)
       INTEGER n,np
       REAL d(np),e(np),z(np,np)
C USES pythag
c QL algorithm with implicit shifts, to determine the eigenvalues and eigenvectors of a real,
c symmetric, tridiagonal matrix, or of a real, symmetric matrix previously reduced by tred2
c §11.2. d is a vector of length np. On input, its first n elements are the diagonal elements of
c the tridiagonal matrix. On output, it returns the eigenvalues. The vector e inputs the subdiagonal
c elements of the tridiagonal matrix, with e(1) arbitrary. On output e is destroyed.
c When finding only the eigenvalues, several lines may be omitted, as noted in the comments.
c If the eigenvectors of a tridiagonal matrix are desired, the matrix z (n by n matrix stored
c in np by np array) is input as the identity matrix. If the eigenvectors of a matrix that has
c been reduced by tred2 are required, then z is input as the matrix output by tred2. In
c either case, the kth column of z returns the normalized eigenvector corresponding to d(k).
       INTEGER i,iter,k,l,m
       REAL b,c,dd,f,g,p,r,s,pythag

       do 11 i=2,n ! Convenient to renumber the elements of e.
         e(i-1)=e(i)
11     enddo
       e(n)=0.
       do 15 l=1,n
         iter=0
1        continue
         do 12 m=l,n-1 ! Look for a single small subdiagonal element
           dd=abs(d(m))+abs(d(m+1)) ! to split the matrix.
           if (abs(e(m))+dd.eq.dd) goto 2
12       enddo 
         m=n
2        continue
         if(m.ne.l)then
           if(iter.eq.30) pause 'too many iterations in tqli'
           iter=iter+1
           g=(d(l+1)-d(l))/(2.*e(l)) !   Form shift.
           r=pythag(g,1.)
           g=d(m)-d(l)+e(l)/(g+sign(r,g))  !  This is dm − ks.
           s=1.
           c=1.
           p=0.
           do 14 i=m-1,l,-1 ! A plane rotation as in the original QL, followed by Givens rotations to restore tridiagonal form.
             f=s*e(i)
             b=c*e(i)
             r=pythag(f,g)
             e(i+1)=r
             if(r.eq.0.)then ! Recover from underflow.
               d(i+1)=d(i+1)-p
               e(m)=0.
               goto 1
             endif
             s=f/r
             c=g/r
             g=d(i+1)-p
             r=(d(i)-g)*s+2.*c*b
             p=s*r
             d(i+1)=g+p
             g=c*r-b 

C   Omit lines from here ...

             do 13 k=1,n ! Form eigenvectors.
               f=z(k,i+1)
               z(k,i+1)=s*z(k,i)+c*f
               z(k,i)=c*z(k,i)-s*f
13           enddo 

C ... to here when finding only eigenvalues.

14         enddo
           d(l)=d(l)-p
           e(l)=g
           e(m)=0.
           goto 1
         endif
15     enddo 
       return
       END


