       SUBROUTINE tred2(a,n,np,d,e)
       INTEGER n,np
       REAL a(np,np),d(np),e(np)
c Householder reduction of a real, symmetric, n by n matrix a, stored in an np by np physical
c array. On output, a is replaced by the orthogonal matrix Q effecting the transformation. d
c returns the diagonal elements of the tridiagonal matrix, and e the off-diagonal elements,
c with e(1)=0. Several statements, as noted in comments, can be omitted if only eigenvalues
c are to be found, in which case a contains no useful information on output. Otherwise they
c are to be included.

       INTEGER i,j,k,l
       REAL f,g,h,hh,scale
       do 18 i=n,2,-1
         l=i-1  
         h=0. 
         scale=0.
         if (l.gt.1) then
           do 11 k=1,l
             scale=scale+abs(a(i,k))
11         enddo
           if (scale.eq.0.) then       !    then Skip transformation.
             e(i)=a(i,l)
           else
             do 12 k=1,l
               a(i,k)=a(i,k)/scale   !     Use scaled a’s for transformation.
               h=h+a(i,k)**2          !    Form σ in h.
12           enddo
             f=a(i,l)
             g=-sign(sqrt(h),f)
             e(i)=scale*g
             h=h-f*g       !      Now h is equation (11.2.4).
             a(i,l)=f-g    !   Store u in the ith row of a.
             f=0.
             do 15 j=1,l
 
C  Omit following line if finding only eigenvalues
 
               a(j,i)=a(i,j)/h   !   Store u/H in ith column of a.
               g=0.               !  Form an element of A · u in g.
               do 13 k=1,j
                 g=g+a(j,k)*a(i,k)
13             enddo 
               do 14 k=j+1,l
                 g=g+a(k,j)*a(i,k)
14             enddo 
               e(j)=g/h       !   Form element of p in temporarily unused
               f=f+e(j)*a(i,j)    !  element of e.
15           enddo 
             hh=f/(h+h)   !   Form K, equation (11.2.11).
             do 17 j=1,l !      Form q and store in e overwriting p.
               f=a(i,j)
               g=e(j)-hh*f
               e(j)=g
               do 16 k=1,j       !  Reduce a, equation (11.2.13).
                 a(j,k)=a(j,k)-f*e(k)-g*a(i,k)
16             enddo 
17           enddo 
           endif
         else
           e(i)=a(i,l)
         endif
         d(i)=h
18     enddo 

C Omit following line if finding only eigenvalues.

       d(1)=0.
       e(1)=0.
       do 24 i=1,n   ! Begin accumulation of transformation matrices.

C Delete lines from here ...

         l=i-1
         if (d(i).ne.0.) then !   This block skipped when i=1.
           do 22 j=1,l
             g=0.
             do 19 k=1,l !  Use u and u/H stored in a to form P · Q.
               g=g+a(i,k)*a(k,j)
19           enddo
             do 21 k=1,l
               a(k,j)=a(k,j)-g*a(k,i)
21           enddo 
22         enddo
         endif

C ... to here when finding only eigenvalues.

         d(i)=a(i,i)  !  This statement remains.

C Also delete lines from here ...

         a(i,i)=1. !  Reset row and column of a to identity matrix for
         do 23 j=1,l     ! next iteration.
           a(i,j)=0.
           a(j,i)=0.
23       enddo

C ... to here when finding only eigenvalues.

24     enddo
       return
       END


