c pc1pc2.f
c Calculate covariance matrix for the princiopal components analysis.
c Miroslav Broz (miroslav.broz@email.cz), Mar 13th 2006

c This is the result of PCA from Nesvorny et al. (2005):
c
c a11 =  0.396; a12 = 0.553; a13 = 0.567; a14 = 0.465;
c a21 = -0.819; a22 = 0.017; a23 = 0.09 ; a24 = 0.567;
c
c PC1 =  a11*(u-g) + a12*(g-r) + a13*(g-i) + a14*(g-z);
c PC2 =  a21*(u-g) + a22*(g-r) + a23*(g-i) + a24*(g-z);

c The solar analog data:
c  u-g = 1.32;
c  g-r = 0.45;
c  r-i = 0.10;
c  i-z = 0.04;
      
      program PC1PC2

      implicit none

      integer MAXREC, MAXDIM
      parameter(MAXREC = 300000)
      parameter(MAXDIM = 5)

      real u, g, r, i_, z
      real uerr, gerr, rerr, ierr, zerr
      real u_g_S, g_r_S, r_i_S, i_z_S
      real u_S, g_S, r_S, i_S, z_S
      real var(MAXDIM, MAXREC)
      real err(MAXDIM, MAXREC)
      real*8 avg(MAXDIM)
      real*8 cov(MAXDIM, MAXDIM)

      integer iu,nrec,ndim,i,j,k,l,ioerr,nlin
      real maxerr, tmp
      character*255 filename
      character*643 line

c functions
      integer length

c solar analog data
      u_g_S = 1.32
      g_r_S = 0.45
      r_i_S = 0.10
      i_z_S = 0.04

      g_S = 0.  ! an arbitrary constant (I will use differences anyway)
      u_S = u_g_S + g_S
      r_S = g_S - g_r_S
      i_S = r_S - r_i_S
      z_S = i_S - i_z_S

c read input parametres
      write(*,70)
70    format('# pc1pc2: Input filename of the Sloan DSS catalogue: ')
      read(*,10,end=990,err=990) filename
10    format(a)

      write(*,80)
80    format('# pc1pc2: Maximum magnitude error for 4 colour indices: ')
      read(*,*,end=990,err=990) maxerr

c read Sloan DSS catalogue colour data
      iu=20
      open(unit=iu,file=filename,form='formatted',status='old',
     :  iostat=ioerr)
      if (ioerr.ne.0) then
        write(*,*) '# pc1pc2: Error reading file ',
     :    filename(1:length(filename)), '.'
        stop
      endif

      i=0
      j=0
      tmp=99.

5     continue
        read(iu,10,end=20,err=20) line
        read(line(164:218),*,end=20,err=20)
     :    u, uerr, g, gerr, r, rerr, i_, ierr, z, zerr

        i=i+1

c select good measurements only
        if (
     :    (u .lt.tmp).and.
     :    (g .lt.tmp).and.
     :    (r .lt.tmp).and.
     :    (i_.lt.tmp).and.
     :    (z .lt.tmp)
     :    ) then

          j=j+1

c subtract the solar analog (this affects only the means!)
          u  = u  - u_S
          g  = g  - g_S
          r  = r  - r_S
          i_ = i_ - i_S
          z  = z  - z_S

c use the color indices instead of colors
          var(1,j) = u-g
          var(2,j) = g-r
          var(3,j) = g-i_
          var(4,j) = g-z

          err(1,j) = sqrt(uerr**2 + gerr**2)
          err(2,j) = sqrt(gerr**2 + rerr**2)
          err(3,j) = sqrt(gerr**2 + ierr**2)
          err(4,j) = sqrt(gerr**2 + zerr**2)

c drop the measurement, if the magnitude error is too large
          if (
     :      (err(1,j).gt.maxerr).or.
     :      (err(2,j).gt.maxerr).or.
     :      (err(3,j).gt.maxerr).or.
     :      (err(4,j).gt.maxerr)
     :      ) then
            j=j-1
          endif

c          var(1,j) = u ! dbg (a test with 5 dimensions)
c          var(2,j) = g
c          var(3,j) = r
c          var(4,j) = i_
c          var(5,j) = z
        endif
      goto 5
20    continue
      close(iu)

      nlin = i
      nrec = j
      ndim = 4
c      ndim = 5 ! dbg

      write(*,60) nlin, filename(1:length(filename)), nrec
60    format('# pc1pc2: Read ', i7, ' records from ', a ,
     :  ', selected ', i7, '.')

c calculate the means...
      do i = 1, ndim
        avg(i) = 0.d0
        do j = 1, nrec
          avg(i) = avg(i) + var(i,j)
        enddo
        avg(i) = avg(i)/nrec

c ... and subtract them
        do j = 1, nrec
          var(i,j) = var(i,j) - avg(i)
        enddo
      enddo

c calculate covariances and the matrix (symmetric)
      do i = 1, ndim
        do j = i, ndim
          cov(i,j) = 0.d0
          do k = 1, nrec

c both avg(i), avg(j) are zero already, so:
            cov(i,j) = cov(i,j) + var(i,k) * var(j,k)

c in case the means were NOT subtracted, use:
c            cov(i,j) = cov(i,j) + (var(i,k)-avg(i)) * (var(j,k)-avg(j))

          enddo
          cov(i,j) = cov(i,j)/(nrec-1)
          cov(j,i) = cov(i,j)   ! the covariance is symmetric
        enddo
      enddo

c print the values and errors to a separate file
      open(unit=iu,file='var.dat',form='formatted',status='unknown',
     :  iostat=ioerr)

      write(iu,100)
100   format('# Colour indices (and their errors) used in the PCA ',
     :  'analysis of the Sloan DSS catalogue.',/,
     :  '# u-g & (u-g)_err & g-r & (g-r)_err & g-i & (g-i)_err & ',
     :  'g-z & (g-z)_err ') 

      do i = 1, nrec
        do j = 1, ndim
          write(iu,90) var(j,i),err(j,i)
90        format(f10.4,1x,f10.4,1x,$)
        enddo
        write(iu,*)
      enddo
      close(iu)

c print the averages (as comments)
      do i = 1, ndim
        write(*,40) i, avg(i)
40      format('# avg(', i1, ') = ', f22.16)
      enddo

c print the matrix (suitable for Matlab input)
      write(*,50)
50    format('# cov(i,j) = ')
      do i = 1, ndim
        do j = 1, ndim
          write(*,30) cov(i,j)
30        format(f22.16,1x,$)
        enddo
        write(*,*)
      enddo

      stop

990   continue
      write(*,*) '# pc1pc2: Error reading standard input.'
      stop

      end

 
